<?php

include_once( 'GelProximity_LifeCycle.php' );
require_once( 'libs/AdminPageFramework_Requirement.php' );
require_once( 'classes/GelProximityConfiguration.php' );
require_once( 'classes/GelProximityFrontPage.php' );
require_once( 'classes/GelProximityUtility.php' );
require_once( 'classes/GelProximityWC.php' );
require_once( 'classes/GelProximityApiS2s.php' );
require_once( 'models/GelProximityCartPrice.php' );
require_once( 'models/GelProximityOrderHistory.php' );

class GelProximity_Plugin extends GelProximity_LifeCycle {

	/**
	 * See: http://plugin.michael-simpson.com/?page_id=31
	 * @return array of option meta data.
	 */
	public function getOptionMetaData() {
		return array();
	}

	protected function getOptionValueI18nString( $optionValue ) {
		$i18nValue = parent::getOptionValueI18nString( $optionValue );

		return $i18nValue;
	}

	protected function initOptions() {
		$options = $this->getOptionMetaData();
		if ( ! empty( $options ) ) {
			foreach ( $options as $key => $arr ) {
				if ( is_array( $arr ) && count( $arr > 1 ) ) {
					$this->addOption( $key, $arr[1] );
				}
			}
		}
	}

	public function getPluginDisplayName() {
		return 'GEL Proximity | La tecnologia per l’ultimo miglio';
	}

	protected function getMainPluginFileName() {
		return 'gelproximity.php';
	}

	/**
	 * See: http://plugin.michael-simpson.com/?page_id=101
	 * Called by install() to create any database tables if needed.
	 * Best Practice:
	 * (1) Prefix all table names with $wpdb->prefix
	 * (2) make table names lower case only
	 * @return void
	 */
	protected function installDatabaseTables() {
		global $wpdb;

		$sql   = array();
		$sql[] = 'CREATE TABLE IF NOT EXISTS `' . $wpdb->prefix . 'gelprx_cart_price' . '` (
    `id_gelprx_cart_price` int(11) NOT NULL AUTO_INCREMENT,
    `id_cart` VARCHAR(255) NOT NULL,
    `response_payload` TEXT NULL DEFAULT NULL,
    `cost` DECIMAL(20,6) NOT NULL DEFAULT \'0.000000\',
    `currency` VARCHAR(10) NULL DEFAULT NULL,
    `date_add` datetime NOT NULL,
    `date_upd` datetime NOT NULL,
    PRIMARY KEY (`id_gelprx_cart_price`),
	INDEX `id_cart` (`id_cart`)
) DEFAULT CHARSET=utf8;';

		$sql[] = 'CREATE TABLE IF NOT EXISTS `' . $wpdb->prefix . 'gelprx_order_history' . '` (
    `id_gelprx_order_history` int(11) NOT NULL AUTO_INCREMENT,
    `id_cart` VARCHAR(255) NOT NULL,
    `id_order` INT(10) NOT NULL,
    `response_success` tinyint(1) unsigned NOT NULL DEFAULT 0,
    `response_id_order` VARCHAR(255) NULL DEFAULT NULL,
    `response_message` TEXT NULL DEFAULT NULL,
    `canceled` tinyint(1) unsigned NOT NULL DEFAULT 0,
    `response_cancel_message` TEXT NULL DEFAULT NULL,
    `date_add` datetime NOT NULL,
    `date_upd` datetime NOT NULL,
    PRIMARY KEY (`id_gelprx_order_history`),
    INDEX `id_order` (`id_order`),
	INDEX `id_cart` (`id_cart`)
) DEFAULT CHARSET=utf8;';

		foreach ( $sql as $query ) {
			$wpdb->query( $query );
		}
	}

	/**
	 * See: http://plugin.michael-simpson.com/?page_id=101
	 * Drop plugin-created tables on uninstall.
	 * @return void
	 */
	protected function unInstallDatabaseTables() {
		if ( 'true' === $this->getOption( 'DropOnUninstall', 'false' ) ) {
			global $wpdb;

			$sql = array(
				'DROP TABLE IF EXISTS `' . $this->prefixTableName( 'gelprx_order_history' ) . '`',
				'DROP TABLE IF EXISTS `' . $this->prefixTableName( 'gelprx_cart_price' ) . '`',
			);

			foreach ( $sql as $query ) {
				$wpdb->query( $query );
			}
		}
	}


	/**
	 * Perform actions when upgrading from version X to version Y
	 * See: http://plugin.michael-simpson.com/?page_id=35
	 * @return void
	 */
	public function upgrade() {
	}

	public function addActionsAndFilters() {
		$_oRequirementCheck = new AdminPageFramework_Requirement(
			array(
				'php'       => array(
					'version' => '5.2.4',
					'error'   => 'The plugin requires the PHP version %1$s or higher.',
				),
				'wordpress' => array(
					'version' => '4.7.0',
					'error'   => 'The plugin requires the WordPress version %1$s or higher.',
				),
				'mysql'     => array(
					'version' => '5.0',
					'error'   => 'The plugin requires the MySQL version %1$s or higher.',
				),
			),
			'GEL Proximity'
		);

		$is_woocommerce_active = false;
		if ( in_array( 'woocommerce/woocommerce.php',
			apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
			$wc_version = GelProximityUtility::get_active_plugin_version( 'woocommerce/woocommerce.php' );
			if ( version_compare( $wc_version, '3.6.0', '>=' ) ) {
				$is_woocommerce_active = true;
			}
		}

		if ( ! $is_woocommerce_active || $_oRequirementCheck->check() ) {
			include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
			if ( is_plugin_active( 'gelproximity/gelproximity.php' ) ) {
				$_oRequirementCheck->deactivatePlugin(
					'gelproximity/gelproximity.php',   // the plugin main file path
					__( 'The plugin requirements are invalid. The plugin has been deactivated.', 'gelproximity' ),
					// additional message
					true    // is in the activation hook. This will exit the script.
				);
			}
		} else {
			add_action( 'wp_enqueue_scripts', array( &$this, 'enqueueStylesAndScripts' ) );
			add_action( 'admin_enqueue_scripts', array( &$this, 'enqueueAdminPageStylesAndScripts' ) );

			// if ( is_admin() ) {

			new GelProximityFrontPage();
			new GelProximityWC();
		}
	}

	public function enqueueAdminPageStylesAndScripts() {
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'gelproximity-custom-admin-script',
			plugins_url( '/js/gelproximity-custom-admin-script.js', __FILE__ ) );

		wp_enqueue_style( 'gelproximity-custom-admin-style',
			plugins_url( '/css/gelproximity-custom-admin-style.css', __FILE__ ) );
	}

	public function enqueueStylesAndScripts() {
		if ( is_page( 'checkout' ) || is_checkout() ) {
			global $woocommerce;
			$id_cart = WC()->session->get( '_gelproximity_order_cart_id' ); // $woocommerce->cart->generate_cart_id();

			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'jquery-effects-core' );
			wp_enqueue_script( 'gelproximity-custom-front-script',
				plugins_url( '/js/gelproximity-custom-front-script.js', __FILE__ ) );

			wp_enqueue_style( 'gelproximity-custom-front-style',
				plugins_url( '/css/gelproximity-custom-front-style.css', __FILE__ ) );

			$id_carrier = GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID;

			$is_sandbox    = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['is_sandbox_mode'];
			$merchant_code = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['merchant_code'];
			$api_key       = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['api_key'];

			$urlEndUser = GelProximityConfiguration::GELPRX_API_URL_SERVER_SANDBOX . '/gel-enduser-client/#/';
			if ( $is_sandbox !== 'yes' ) {
				$urlEndUser = GelProximityConfiguration::GELPRX_API_URL_SERVER_PRODUCTION . '/gel-enduser-client/#/';
			}

			wp_localize_script( 'gelproximity-custom-front-script', 'gelprx_conf', array(
				'id_carrier'              => $id_carrier,
				'ajax_url'                => admin_url( 'admin-ajax.php' ),
				'ajax_token'              => GelProximityConfiguration::GELPRX_CONFIG_AJAX_TOKEN,
				'nonce'                   => wp_create_nonce( 'gelproximity' ),
				'isSandboxMode'           => $is_sandbox === 'yes',
				'apiKey'                  => $api_key,
				'merchantCode'            => $merchant_code,
				'reference'               => GelProximityUtility::buildOrderReference( $id_cart ),
				'redirectMode'            => GelProximityConfiguration::GELPRX_API_REDIRECT_MODE,
				'locale'                  => ! empty( $locale = get_user_locale() ) ? $locale : GelProximityConfiguration::GELPRX_CONFIG_DEFAULT_LOCALE,
				'urlRedirectOk'           => get_home_url() . '/wp-json/gelproximity/handle?status=ok',
				// Non funziona per modalità modal
				'urlRedirectCancel'       => get_home_url() . '/wp-json/gelproximity/handle?status=cancel',
				// Non funziona per modalità modal
				'urlEndUser'              => $urlEndUser,
				'isReturnOrder'           => false,
				'elementSelector'         => 'gelproximity-app__selection-button-sdk-wrapper',
				'buttonText'              => __( 'Select the pick-up point', 'gelproximity' ),
				'buttonEditText'          => __( 'Change the pick-up point', 'gelproximity' ),
				'defaultCarrierCostLabel' => '',
				// __('Select the pick-up point'),
			) );

			wp_enqueue_script( 'gelproximity-sdk', 'https://code.gelproximity.com/sdk/latest.js', array(), false, true );
		}

		// Cart Page
		if ( is_page( 'cart' ) || is_cart() ) {
			wp_enqueue_style( 'gelproximity-custom-front-style',
				plugins_url( '/css/gelproximity-custom-front-style.css', __FILE__ ) );
		}
	}

}
