<?php

class GelProximityFrontPage {

	public function __construct() {
		add_action( 'rest_api_init', array( &$this, 'gelproximity_wpc_register_wp_api_endpoints' ) );
		add_action( 'wp_ajax_gelproximity_set_pickup_point', array( &$this, 'gelproximity_set_pickup_point' ) );
		add_action( 'wp_ajax_nopriv_gelproximity_set_pickup_point', array( &$this, 'gelproximity_set_pickup_point' ) );

		add_filter( 'safe_style_css', function( $styles ) {
			$styles[] = 'display';
			return $styles;
		} );
	}

	// /wp-json/gelproximity/handle
	public function gelproximity_wpc_register_wp_api_endpoints() {
		if ( function_exists( 'register_rest_route' ) ) {
			register_rest_route( 'gelproximity', '/handle', array(
				'methods'             => array( 'GET', 'POST' ),
				'callback'            => array( &$this, 'gelproximity_wpc_somename_search_callback' ),
				'permission_callback' => '__return_true',
			) );
		}
	}

	public function gelproximity_wpc_somename_search_callback( $request_data ) {
		return array( 'ok' );
	}

	public function gelproximity_set_pickup_point() {
		$id_cart = WC()->session->get( '_gelproximity_order_cart_id' ); // $woocommerce->cart->generate_cart_id();

		$response = array();
		$payload  = $_POST['payload'];

		if ( empty( $id_cart ) || empty( $payload ) ) {
			// Renderizza da template per riepilogo
			$tpl_file           = plugin_dir_path( __DIR__ ) . 'templates/carrier-pick-up-point-details-error.tpl';
			$tpl_params         = array(
				'{{$error_message}}' => __( 'An error occurred while selecting the pickup location, please try again.',
					'gelproximity' ),
			);
			$html               = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );
			$response["status"] = "error";
			$response["data"]   = array( 'html' => $html );
			wp_send_json( $response );
			wp_die();
		}

		GelProximityUtility::doLog( $payload, 'gelproximity_set_pickup_point' );
		GelProximityCartPrice::storeOrUpdate( $id_cart, $payload );

		// Renderizza da template per riepilogo
		$tpl_file   = plugin_dir_path( __DIR__ ) . 'templates/carrier-pick-up-point-details.tpl';
		$tpl_params = array(
			'{{$payload_pickupPointId}}' => $payload['pickupPointId'],
			'{{$payload_code}}'          => $payload['code'],
			'{{$label_1}}'               => __( 'Pick-up point selected', 'gelproximity' ),
			'{{$payload_address}}'       => $payload['address'],
			'{{$payload_city}}'          => $payload['city'],
			'{{$payload_department}}'    => $payload['department'],
			'{{$payload_zipCode}}'       => $payload['zipCode'],
			'{{$payload_country}}'       => $payload['country'],

		);
		$html       = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );

		$vat_included = '(' . __( 'Tax incl.', 'gelproximity' ) . ')';

		$cost = GelProximityUtility::formatEurPrice( $payload['cost'] ) . ' ' . $vat_included;

		$response["status"] = "ok";
		$response["data"]   = array( 'html' => $html, 'cost' => $cost );
		wp_send_json( $response );
		wp_die();
	}

	/**
	 * Checks if the current request is a WP REST API request.
	 *
	 * Case #1: After WP_REST_Request initialisation
	 * Case #2: Support "plain" permalink settings
	 * Case #3: It can happen that WP_Rewrite is not yet initialized,
	 *          so do this (wp-settings.php)
	 * Case #4: URL Path begins with wp-json/ (your REST prefix)
	 *          Also supports WP installations in subfolders
	 *
	 * @returns boolean
	 * @author matzeeable
	 */
	public function is_rest() {
		$prefix = rest_get_url_prefix();
		if ( ( defined( 'REST_REQUEST' ) && REST_REQUEST ) // (#1)
		     || ( isset( $_GET['rest_route'] ) // (#2)
		          && strpos( trim( $_GET['rest_route'], '\\/' ), $prefix, 0 ) === 0 ) ) {
			return true;
		}
		// (#3)
		global $wp_rewrite;
		if ( $wp_rewrite === null ) {
			$wp_rewrite = new WP_Rewrite();
		}

		// (#4)
		$rest_url    = wp_parse_url( trailingslashit( rest_url() ) );
		$current_url = wp_parse_url( add_query_arg( array() ) );

		return strpos( $current_url['path'], $rest_url['path'], 0 ) === 0;
	}

	private function in_array_match( $value, $array ) {
		foreach ( $array as $k => $v ) {
			if ( strpos( $value, trim( $v ) ) !== false ) {
				return true;
			}
		}

		return false;
	}
}