<?php

class GelProximityUtility {
	/**
	 * Determines if a post, identified by the specified ID, exist
	 * within the WordPress database.
	 *
	 * @param int $id The ID of the post to check
	 *
	 * @return   bool          True if the post exists; otherwise, false.
	 * @since    1.0.0
	 */
	public static function post_exists( $id ) {
		return is_string( get_post_status( $id ) );
	}

	public static function get_pages_array() {
		$pages    = array();
		$page_ids = get_all_page_ids();
		foreach ( $page_ids as $id ) {
			$pages[ $id ] = get_the_title( $id );
		}

		return $pages;
	}

	public static function doSerialize( $obj, $type = 'json' ) {
		if ( $type === 'json' ) {
			return json_encode(
				$obj,
				JSON_HEX_TAG | JSON_HEX_APOS | JSON_HEX_QUOT | JSON_HEX_AMP | JSON_UNESCAPED_UNICODE
			);
		}

		return false;
	}

	public static function doUnserialize( $strobj, $type = 'json', $params = '' ) {
		if ( $type === 'json' ) {
			return json_decode( $strobj, isset( $params['assoc'] ) ? $params['assoc'] : false );
		}

		return false;
	}

	public static function get_active_plugin_version( $plugin_path_file, $sitewide = false ) {
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		$plugins = get_plugins();
		if ( isset( $plugins[ $plugin_path_file ] ) ) {
			return $plugins[ $plugin_path_file ]['Version'];
		}

		return false;
	}

	public static function buildOrderReference( $reference ) {
		// return GelProximityConfiguration::GELPRX_CONFIG_API_REFERENCE_PREFIX . $reference;
		return $reference;
	}

	public static function formatEurPrice( $price ) {
		return str_replace( '.', ',', $price ) . ' €';
	}

	public static function renderTemplate( $tpl_params, $tpl_file ) {
		$template = file_get_contents( $tpl_file );

		foreach ( $tpl_params as $bookmark => $text ) {
			$template = str_replace( $bookmark, $text, $template );
		}

		return $template;
	}

	public static function getHtmlAllowedTags() {
		$default_attribs = array(
			'id'    => array(),
			'type'  => array(),
			'class' => array(),
			'title' => array(),
			'style' => array(),
			'data'  => array(),
			'name'  => array(),
			'value' => array(),
		);

		$allowed_tags = array(
			'div'        => $default_attribs,
			'button'     => $default_attribs,
			'span'       => $default_attribs,
			'p'          => $default_attribs,
			'a'          => $default_attribs,
			'u'          => $default_attribs,
			'i'          => $default_attribs,
			'q'          => $default_attribs,
			'b'          => $default_attribs,
			'ul'         => $default_attribs,
			'ol'         => $default_attribs,
			'li'         => $default_attribs,
			'br'         => $default_attribs,
			'hr'         => $default_attribs,
			'strong'     => $default_attribs,
			'blockquote' => $default_attribs,
			'del'        => $default_attribs,
			'strike'     => $default_attribs,
			'em'         => $default_attribs,
			'code'       => $default_attribs,
			'table'      => $default_attribs,
			'input'      => $default_attribs,
			'tr'         => $default_attribs,
			'td'         => $default_attribs,
			'h1'         => $default_attribs,
			'h2'         => $default_attribs,
			'h3'         => $default_attribs,
			'h4'         => $default_attribs,
			'h5'         => $default_attribs,
			'h6'         => $default_attribs,
		);

		return $allowed_tags;
	}

	// Log in separate file for debug
	public static function doLog( $text, $context = '', $prefix = 'debug', $subdir = '' ) {
		// Enable logs only
		if ( ! GelProximityConfiguration::GELPRX_CONFIG_LOG_ENABLED ) {
			return false;
		}

		$dir = plugin_dir_path( __DIR__ ) . 'logs';

		if ( ! is_dir( $dir ) ) {
			if ( ! mkdir( $dir, 0755 ) && ! is_dir( $dir ) ) {
				return false;
			}
		}

		if ( ! file_exists( $dir ) ) {
			return false;
		}

		if ( ! empty( $subdir ) ) {
			$subdir = $dir . '/' . $subdir;
			if ( ! is_dir( $subdir ) ) {
				if ( ! mkdir( $subdir, 0755 ) && ! is_dir( $subdir ) ) {
					return false;
				}
			}

			if ( ! file_exists( $subdir ) ) {
				return false;
			}

			$dir .= '/' . $subdir;
		}

		$timestamp = date( "Ymd" );
		$filename  = $dir . DIRECTORY_SEPARATOR . $prefix . "_log_" . $timestamp . ".log";

		// Get only object vars
		if ( is_object( $text ) ) {
			$text = get_object_vars( $text );
		}

		// open log file
		$fh = fopen( $filename, "a" );
		fwrite( $fh, date( "Y-m-d, H:i:s" ) . "\n" );
		if ( ! empty( $context ) ) {
			fwrite( $fh, '*** CONTEXT: ' . $context . ' ***' . "\n" );
		}
		fwrite( $fh, print_r( $text, true ) . "\n" );
		fclose( $fh );
	}

	public static function enable_php_errors() {
		ini_set( 'display_errors', 1 );
		ini_set( 'display_startup_errors', 1 );
		error_reporting( E_ALL );
	}
}