<?php

class GelProximityWC {
	public function __construct() {
		$wc_version = GelProximityUtility::get_active_plugin_version( 'woocommerce/woocommerce.php' );

		add_action( 'woocommerce_shipping_init', array( &$this, 'gelproximity_shipping_method_init' ) );

		add_filter( 'woocommerce_shipping_methods', array( &$this, 'gelproximity_add_shipping_method' ) );

		add_filter( 'woocommerce_shipping_method_description', array( &$this, 'gelproximity_filter_function_name' ), 10,
			2 );

		// Adding Meta container admin shop_order pages
		add_action( 'add_meta_boxes', array( &$this, 'gelproximity_add_meta_boxes' ) );

		// Refresh cached shipping methods on checkout update ajax event in Woocommerce
		add_action( 'woocommerce_checkout_update_order_review',
			array( &$this, 'gelproximity_checkout_update_refresh_shipping_methods' ), 10, 1 );

		// Refresh cached shipping methods on cart update ajax event in Woocommerce
		add_filter( 'woocommerce_after_calculate_totals',
			array( &$this, 'gelproximity_checkout_update_refresh_shipping_methods' ),
			10, 2 );

		add_action( 'woocommerce_after_shipping_rate', array( &$this, 'gelproximity_action_after_shipping_rate' ), 20,
			2 );

		add_action( 'woocommerce_checkout_create_order', array( &$this, 'gelproximity_before_checkout_create_order' ),
			20, 2 );

		add_action( 'woocommerce_add_to_cart', array( &$this, 'gelproximity_custome_add_to_cart' ) );

		add_action( 'woocommerce_after_checkout_validation', array( &$this, 'gelproximity_validate_checkout' ), 10, 2 );

		if ( version_compare( $wc_version, '3.7.0', '>=' ) ) {
			add_action( 'woocommerce_before_thankyou', array( &$this, 'gelproximity_woocommerce_thankyou' ), 4 );
		} else {
			add_action( 'woocommerce_thankyou', array( &$this, 'gelproximity_woocommerce_thankyou' ), 4 );
		}

		add_action( 'woocommerce_order_status_changed',
			array( &$this, 'gelproximity_woocommerce_order_status_changed' ), 10, 3 );

		add_action( 'save_post', array( &$this, 'gelproximity_save_meta_box' ) );

		add_action( 'http_api_curl', array( &$this, 'custom_action_http_api_curl' ), 9999, 3 );
	}

	public function custom_action_http_api_curl( &$handle, $r, $url ) {
		$is_sandbox = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['is_sandbox_mode'];
		$urlEndUser = GelProximityConfiguration::GELPRX_API_URL_SERVER_SANDBOX;
		if ( $is_sandbox !== 'yes' ) {
			$urlEndUser = GelProximityConfiguration::GELPRX_API_URL_SERVER_PRODUCTION;
		}

		if ( strpos( $url, $urlEndUser ) !== false ) {
			$certificate = plugin_dir_path( __DIR__ ) . 'cacert.pem';
			curl_setopt( $handle, CURLOPT_CAINFO, $certificate );
			curl_setopt( $handle, CURLOPT_CAPATH, $certificate );
		}
	}

	public function gelproximity_save_meta_box( $post_id ) {
		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return;
		}
		if ( $parent_id = wp_is_post_revision( $post_id ) ) {
			$post_id = $parent_id;
		}

		if ( isset( $_POST['gel_prx_id_order'] ) && isset( $_POST['gel_proximity_order_admin_page_asking_retry'] ) ) {
			$gel_prx_id_order = (int) sanitize_text_field( $_POST['gel_prx_id_order'] );
			// Try to send confirmOrder again
			GelProximityWCShippingMethod::doConfirmOrder( $gel_prx_id_order, true );
		}
	}

	public function gelproximity_woocommerce_order_status_changed( $order_id, $old_status, $new_status ) {
		$order = wc_get_order( $order_id );
		// Iterating through order shipping items
		foreach ( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ) {
			$shipping_method_id = $shipping_item_obj->get_method_id(); // The method ID
			if ( $shipping_method_id === GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID ) {
				$order_status_paid_arr = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['confirm_order_statuses'];

				// Add order statuses without wc- prefix
				$order_status_paid_arr_no_prefix = array();
				foreach ( $order_status_paid_arr as $order_state ) {
					if ( strpos( $order_state, 'wc-' ) === 0 ) {
						$order_status_paid_arr_no_prefix[] = str_replace( 'wc-', '', $order_state );
					}
				}

				if ( $old_status !== $new_status && ( in_array( $new_status, $order_status_paid_arr,
							true ) || in_array( $new_status, $order_status_paid_arr_no_prefix, true ) ) ) {
					GelProximityWCShippingMethod::doConfirmOrder( $order_id, false );
				}
				break;
			}
		}
	}

	public function gelproximity_woocommerce_thankyou( $order_id ) {
		$order = wc_get_order( $order_id );
		// Iterating through order shipping items
		foreach ( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ) {
			$shipping_method_id = $shipping_item_obj->get_method_id(); // The method ID

			if ( $shipping_method_id === GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID ) {
				$payload = GelProximityCartPrice::getPayloadByPsIdOrder( $order_id );

				if ( ! empty( $payload ) ) {
					$payload    = GelProximityUtility::doUnserialize( $payload, 'json', array( 'assoc' => true ) );
					$tpl_file   = plugin_dir_path( __DIR__ ) . 'templates/carrier-pick-up-point-details.tpl';
					$tpl_params = array(
						'{{$payload_pickupPointId}}' => esc_html( $payload['pickupPointId'] ),
						'{{$payload_code}}'          => esc_html( $payload['code'] ),
						'{{$label_1}}'               => __( 'Pick-up point selected', 'gelproximity' ),
						'{{$payload_address}}'       => esc_html( $payload['address'] ),
						'{{$payload_city}}'          => esc_html( $payload['city'] ),
						'{{$payload_department}}'    => esc_html( $payload['department'] ),
						'{{$payload_zipCode}}'       => esc_html( $payload['zipCode'] ),
						'{{$payload_country}}'       => esc_html( $payload['country'] ),

					);
					$html       = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );

					echo wp_kses( $html, GelProximityUtility::getHtmlAllowedTags() );
				}

				break;
			}
		}
	}

	public function gelproximity_validate_checkout( $fields, $errors ) {
		// Check if a pick-up point was selected
		$is_pickup_point_selected = false;
		$id_cart                  = WC()->session->get( '_gelproximity_order_cart_id' ); // $woocommerce->cart->generate_cart_id();
		if ( ! empty( $id_cart ) ) {
			$payload = GelProximityCartPrice::getPayloadByPsIdCart( $id_cart );
			if ( ! empty( $payload ) ) {
				$is_pickup_point_selected = true;
			}
		}

		if ( isset( $fields['shipping_method'] ) ) {
			if ( is_array( $fields['shipping_method'] ) ) {
				foreach ( $fields['shipping_method'] as $shipping_method ) {
					if ( $shipping_method === GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID ) {
						if ( ! $is_pickup_point_selected ) {
							$errors->add( 'validation',
								__( 'You must select a pick-up point before proceding.', 'gelproximity' ) );
						}
					}
				}
			} elseif ( $fields['shipping_method'] === GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID ) {
				if ( ! $is_pickup_point_selected ) {
					$errors->add( 'validation',
						__( 'You must select a pick-up point before proceding.', 'gelproximity' ) );
				}
			}
		}
	}

	public function gelproximity_custome_add_to_cart() {
		$gelproximity_order_cart_id = WC()->session->get( '_gelproximity_order_cart_id' );
		if ( empty( $gelproximity_order_cart_id ) ) {
			$uuid = wp_generate_uuid4();
			WC()->session->set( '_gelproximity_order_cart_id', wp_generate_uuid4() );
			GelProximityUtility::doLog( $uuid, 'UUID Cart Generated: gelproximity_custome_add_to_cart' );
		}
	}

	public function gelproximity_before_checkout_create_order( $order, $data ) {
		$session_var  = '_gelproximity_order_cart_id';
		$session_data = WC()->session->get( $session_var );
		if ( ! empty( $session_data ) ) {
			$order->update_meta_data( '_gelproximity_order_cart_id', $session_data );
		} else {
			// error_log( "no session data", 0 );
		}
		WC()->session->set( '_gelproximity_order_cart_id', false );
	}

	public function gelproximity_action_after_shipping_rate( $method, $index ) {
		// Targeting checkout page only:
		if ( is_cart() ) {
			return;
		} // Exit on cart page

		if ( $method->id === GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID ) {
			// Fill gelproximity-app__selection-details with last selection
			global $woocommerce;
			$id_cart = WC()->session->get( '_gelproximity_order_cart_id' ); // $woocommerce->cart->generate_cart_id();

			if ( empty( $id_cart ) ) {
				$uuid = wp_generate_uuid4();
				WC()->session->set( '_gelproximity_order_cart_id', wp_generate_uuid4() );
				GelProximityUtility::doLog( $uuid, 'UUID Cart Generated: gelproximity_action_after_shipping_rate' );
				$id_cart = WC()->session->get( '_gelproximity_order_cart_id' );
			}

			$html = '';
			if ( ! empty( $id_cart ) ) {
				$payload = GelProximityCartPrice::getPayloadByPsIdCart( $id_cart );
				if ( ! empty( $payload ) ) {
					$payload = GelProximityUtility::doUnserialize( $payload, 'json', array( 'assoc' => true ) );
					// Renderizza da template per riepilogo
					$tpl_file   = plugin_dir_path( __DIR__ ) . 'templates/carrier-pick-up-point-details.tpl';
					$tpl_params = array(
						'{{$payload_pickupPointId}}' => esc_html( $payload['pickupPointId'] ),
						'{{$payload_code}}'          => esc_html( $payload['code'] ),
						'{{$label_1}}'               => __( 'Pick-up point selected', 'gelproximity' ),
						'{{$payload_address}}'       => esc_html( $payload['address'] ),
						'{{$payload_city}}'          => esc_html( $payload['city'] ),
						'{{$payload_department}}'    => esc_html( $payload['department'] ),
						'{{$payload_zipCode}}'       => esc_html( $payload['zipCode'] ),
						'{{$payload_country}}'       => esc_html( $payload['country'] ),

					);
					$html       = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );
				}
			}

			$button_pickup_point_label = __( 'Select the pick-up point', 'gelproximity' );
			if ( ! empty( $html ) ) {
				$button_pickup_point_label = __( 'Change the pick-up point', 'gelproximity' );
			}

			// Show button to select the pick-up point
			$tpl_file   = plugin_dir_path( __DIR__ ) . 'templates/carrier-pick-up-point-wrapper.tpl';
			$tpl_params = array(
				'{{$button_pickup_point_label}}' => esc_html( $button_pickup_point_label ),
				'{{$id_cart}}'                   => esc_html( $id_cart ),
				'{{$html}}'                      => $html,

			);
			$html_full  = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );

			echo wp_kses( $html_full, GelProximityUtility::getHtmlAllowedTags() );
		}
	}

	public function gelproximity_checkout_update_refresh_shipping_methods( $post_data ) {
		// TODO: clear cache if another pick-up point is selected
		$packages = WC()->cart->get_shipping_packages();
		foreach ( $packages as $package_key => $package ) {
			WC()->session->set( 'shipping_for_package_' . $package_key, false ); // Or true
		}
	}

	public function gelproximity_shipping_method_init() {
		include_once( 'GelProximityWCShippingMethod.php' );
		if ( ! class_exists( 'GelProximityWCShippingMethod' ) ) {
			new GelProximityWCShippingMethod();
		}
	}

	public function gelproximity_add_shipping_method( $methods ) {
		$methods['gelproximity_shipping_method'] = 'GelProximityWCShippingMethod';

		return $methods;
	}

	// woocommerce_shipping_method_description
	public function gelproximity_filter_function_name( $method_description, $that ) {
		if ( is_admin() && strpos( $_SERVER['REQUEST_URI'], 'section=gelproximity_shipping_method' ) !== false ) {
			return $this->display_plugin_header_info(); // $method_description;
		}
	}

	public function gelproximity_add_meta_boxes() {
		// Check if order is shipped with this carrier
		global $post;
		$order_id = $post->ID;

		if ( get_post_type( $post->ID ) === 'shop_order' ) {
			// An instance of
			$order = wc_get_order( $order_id );

			// Iterating through order shipping items
			foreach ( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ) {
				// $order_item_name             = $shipping_item_obj->get_name();
				// $order_item_type             = $shipping_item_obj->get_type();
				// $shipping_method_title       = $shipping_item_obj->get_method_title();
				$shipping_method_id = $shipping_item_obj->get_method_id(); // The method ID
				// $shipping_method_instance_id = $shipping_item_obj->get_instance_id(); // The instance ID
				// $shipping_method_total       = $shipping_item_obj->get_total();
				// $shipping_method_total_tax   = $shipping_item_obj->get_total_tax();
				// $shipping_method_taxes       = $shipping_item_obj->get_taxes();

				// $id_cart = get_post_meta( $order_id, '_gelproximity_order_cart_id', true ); // $order->get_cart_hash();

				if ( $shipping_method_id === GelProximityConfiguration::GELPRX_CONFIG_CARRIER_ID ) {
					add_meta_box( 'gelproximity_order_info', __( 'GEL Proximity Order', 'gelproximity' ),
						array( &$this, 'gelproximity_add_order_info_box' ), 'shop_order', 'side', 'core' );
					break;
				}
			}
		}
	}

	public function gelproximity_add_order_info_box() {
		global $post;
		$order_id = $post->ID;
		$id_cart  = get_post_meta( $order_id, '_gelproximity_order_cart_id', true );
		$payload  = GelProximityCartPrice::getPayloadByPsIdOrder( $order_id );

		$tpl_params = array(
			'{{$column_name}}'  => __( 'Name', 'gelproximity' ),
			'{{$column_value}}' => __( 'Value', 'gelproximity' ),
		);

		if ( ! empty( $id_cart ) && ! empty( $payload ) ) {
			$payload = GelProximityUtility::doUnserialize( $payload, 'json', array( 'assoc' => true ) );

			$response_status_class = 'warning';
			$response_status       = __( 'Waiting order confirmation', 'gelproximity' );
			$response_message      = '';
			$is_retry_enabled      = false;
			$gelprx_order_history  = GelProximityOrderHistory::getAllByPsIdCart( $id_cart );
			if ( ! empty( $gelprx_order_history ) ) {
				if ( (int) $gelprx_order_history['response_success'] === 1 ) {
					$response_status_class = 'success';
					$response_status       = __( 'Success', 'gelproximity' );
					$is_retry_enabled      = false;
				} else {
					$response_status_class = 'error';
					$response_status       = __( 'Error', 'gelproximity' );
					$is_retry_enabled      = true;
					$response_message      = $gelprx_order_history['response_message'];
				}
			}

			$tpl_file = plugin_dir_path( __DIR__ ) . 'templates/carrier-meta-box-content.tpl';
			if ( $is_retry_enabled ) {
				$tpl_file = plugin_dir_path( __DIR__ ) . 'templates/carrier-meta-box-content-retry.tpl';
			}
			$tpl_params_extra = array(
				'{{$id_order}}'                      => esc_html( $order_id ),
				'{{$column_name_Reference}}'         => __( 'Reference', 'gelproximity' ),
				'{{$column_value_Reference}}'        => esc_html( $payload['reference'] ),
				'{{$column_name_PickupPointId}}'     => __( 'Pickup Point ID', 'gelproximity' ),
				'{{$column_value_PickupPointId}}'    => esc_html( $payload['pickupPointId'] ),
				'{{$column_name_Code}}'              => __( 'Code', 'gelproximity' ),
				'{{$column_value_Code}}'             => esc_html( $payload['code'] ),
				'{{$column_name_Address}}'           => __( 'Address', 'gelproximity' ),
				'{{$column_value_Address}}'          => esc_html( $payload['address'] ),
				'{{$column_name_City}}'              => __( 'City', 'gelproximity' ),
				'{{$column_value_City}}'             => esc_html( $payload['city'] ),
				'{{$column_name_Department}}'        => __( 'Department', 'gelproximity' ),
				'{{$column_value_Department}}'       => esc_html( $payload['department'] ),
				'{{$column_name_ZipCode}}'           => __( 'Zip code', 'gelproximity' ),
				'{{$column_value_ZipCode}}'          => esc_html( $payload['zipCode'] ),
				'{{$column_name_Country}}'           => __( 'Country', 'gelproximity' ),
				'{{$column_value_Country}}'          => esc_html( $payload['country'] ),
				'{{$column_name_Status}}'            => __( 'Status', 'gelproximity' ),
				'{{$column_value_Status_class}}'     => esc_html( $response_status_class ),
				'{{$column_value_Status}}'           => esc_html( $response_status ),
				'{{$column_name_ErrorDetail_class}}' => esc_html( $response_status_class ) === 'error' ? '' : 'gelproximity-app__hidden',
				'{{$column_name_ErrorDetail}}'       => __( 'Detail', 'gelproximity' ),
				'{{$column_value_ErrorDetail}}'      => esc_html( $response_message ),
				'{{$button_label_retry}}'            => __( 'Retry send order', 'gelproximity' ),
			);
			$tpl_params       = array_merge( $tpl_params, $tpl_params_extra );
			$html_full        = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );
			echo wp_kses( $html_full, GelProximityUtility::getHtmlAllowedTags() );
		} else {
			$tpl_file  = plugin_dir_path( __DIR__ ) . 'templates/carrier-meta-box-content-empty.tpl';
			$html_full = GelProximityUtility::renderTemplate( $tpl_params, $tpl_file );
			echo wp_kses( $html_full, GelProximityUtility::getHtmlAllowedTags() );
		}
	}

	public function display_plugin_header_info() {
		$url  = 'https://gelproximity.com/';
		$logo = GELPRX_ASSETS_URL . '/img/gelproximity_logo_admin.png';

		return '<div class="gelproximity_display_plugin_header_info_wrapper">
			<a href="' . $url . '" target="_blank"><img src="' . $logo . '"/></a>
			<p class="gelproximity_display_plugin_header_info_description">' . __( 'Offer your customers an alternative logistics service to home delivery! Use the over 30,000 authorized collection points in Italy,
improve the UX of your customers at checkout, add your stores and start shipping nearby in Pay per Ship mode.',
				'gelproximity' ) . '</p></div>';
	}
}