<?php

class GelProximityWCShippingMethod extends WC_Shipping_Method {
	public $wc_order_statuses = array();

	/**
	 * Constructor for your shipping class
	 *
	 * @access public
	 * @return void
	 */
	public function __construct() {
		$this->id                 = 'gelproximity_shipping_method'; // Id for your shipping method. Should be unique.
		$this->method_title       = __( 'GEL Proximity', 'gelproximity' );  // Title shown in admin // Punto di ritiro
		$this->method_description = __( 'Select where to collect your order in 24-48h!',
			'gelproximity' ); // Description shown in admin // Scegli dove ritirare il tuo ordine!

		// Availability & Countries
		$this->availability = 'including';
		$this->countries    = array(
			'IT', // Italy
		);
		$this->tax_status   = 'none'; // Ignore taxes on shipping fee

		if ( function_exists( 'wc_get_order_statuses' ) ) {
			foreach ( wc_get_order_statuses() as $order_key => $order_state_label ) {
				$this->wc_order_statuses[ $order_key ] = $order_state_label;
			}
		}

		$this->init();

		$this->enabled = isset( $this->settings['enabled'] ) ? $this->settings['enabled'] : 'yes'; // This can be added as an setting but for this example its forced enabled
		$this->title   = __( 'Pick-up point',
				'gelproximity' ) . ' - ' . __( 'Select where to collect your order in 24-48h!',
				'gelproximity' ); // This can be added as an setting but for this example its forced.
	}

	/**
	 * Init your settings
	 *
	 * @access public
	 * @return void
	 */
	public function init() {
		// Load the settings API
		$this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
		$this->init_settings(); // This is part of the settings API. Loads settings you previously init.

		// Save settings in admin if you have any defined
		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
	}

	/**
	 * Define settings field for this shipping
	 * @return void
	 */
	public function init_form_fields() {
		$this->form_fields = array(
			'enabled'                => array(
				'title'       => __( 'Enable', 'gelproximity' ),
				'type'        => 'checkbox',
				'description' => __( 'Enable this shipping.', 'gelproximity' ),
				'default'     => 'yes',
			),
			'is_sandbox_mode'        => array(
				'title'       => __( 'Sandbox mode', 'gelproximity' ),
				'type'        => 'checkbox',
				'description' => __( 'Make a try! Enable the sandbox mode to test the service.', 'gelproximity' ),
				'desc_tip'    => false,
				'default'     => 'yes',
			),
			'api_key'                => array(
				'title'       => __( 'API Key', 'gelproximity' ),
				'type'        => 'text',
				'description' => __( 'Please enter your API key here', 'gelproximity' ),
				'default'     => '',
			),
			'api_key_s2s'            => array(
				'title'       => __( 'API Key Server', 'gelproximity' ),
				'type'        => 'text',
				'description' => __( 'Please enter your API key Server to Server here', 'gelproximity' ),
				'default'     => '',
			),
			'merchant_code'          => array(
				'title'       => __( 'Merchant Code', 'gelproximity' ),
				'type'        => 'text',
				'description' => __( 'Please enter your merchant code here', 'gelproximity' ),
				'default'     => '',
			),
			'default_cost'           => array(
				'title'       => __( 'Default carrier cost', 'gelproximity' ),
				'type'        => 'price',
				'description' => '<p>' . __( 'This is the shipping price shown before selecting a specific collection point, excluding VAT.',
						'gelproximity' ) . '</p>' .
				                 '<p>' . __( 'If you have set a single price for all networks, please also report it here. If you have defined different prices, please report the minimum value.',
						'gelproximity' ) . '</p>',
				'default'     => 0,
			),
			'confirm_order_statuses' => array(
				'title'       => __( 'Confirm order Statuses', 'gelproximity' ),
				'description' => __( 'Select the states that determine a confirmed order. Once confirmed, the order will appear on the GEL Proximity platform.',
					'gelproximity' ),
				'type'        => 'multiselect',
				'default'     => '',
				'options'     => $this->wc_order_statuses,
			),

		);
	}

	/**
	 * This function is used to calculate the shipping cost. Within this function we can check for weights, dimensions and other parameters.
	 *
	 * @access public
	 *
	 * @param mixed $package
	 *
	 * @return void
	 */
	public function calculate_shipping( $package = array() ) {
		if ( is_page( 'cart' ) || is_cart() ) {
			// In the cart don't show the cost
			$rate = array(
				'id'    => $this->id,
				'label' => $this->title,
				'cost'  => 0,
				// 'calc_tax' => 'per_item'
			);

			$this->add_rate( $rate );
		} else {
			// Get default cost
			$cost = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['default_cost'];
			if ( ! empty( $cost ) ) {
				$cost = 0; // str_replace( ',', '.', $cost ) * GelProximityConfiguration::GELPRX_DEFAULT_VAT; // TODO
			} else {
				$cost = 0;
			}

			// Check if a pick-up point was selected
			$id_cart = WC()->session->get( '_gelproximity_order_cart_id' ); // $woocommerce->cart->generate_cart_id();

			if ( ! empty( $id_cart ) ) {
				$payload = GelProximityCartPrice::getPayloadByPsIdCart( $id_cart );
				if ( ! empty( $payload ) ) {
					$payload = GelProximityUtility::doUnserialize( $payload, 'json', array( 'assoc' => true ) );
					$cost    = $payload['cost'];
				}
			}

			$rate = array(
				'id'    => $this->id,
				'label' => $this->title,
				'cost'  => $cost,
				// 'calc_tax' => 'per_item'
			);

			$this->add_rate( $rate );
		}
	}
	// + Fields validation

	/**
	 * @throws Exception
	 */
	public function validate_field_empty( &$key, &$value ) {
		if ( empty( $value ) ) {
			// WC_Admin_Settings::add_error( "TestError: Key:{$key}::Value:{$value}" ); // Used for debugging
			WC_Admin_Settings::add_error( sprintf( __( 'The %1$s value must not be empty', 'gelproximity' ), $key ) );
			throw new Exception( "Error: Key:{$key}::Value:{$value}" );
		} else {
			return $value;
		}
	}

	public function validate_api_key_field( $key, $value ) {
		return $this->validate_field_empty( $key, $value );
	}

	public function validate_api_key_s2s_field( $key, $value ) {
		return $this->validate_field_empty( $key, $value );
	}

	public function validate_merchant_code_field( $key, $value ) {
		return $this->validate_field_empty( $key, $value );
	}

	public function validate_default_cost_field( $key, $value ) {
		return $this->validate_field_empty( $key, $value );
	}

	public function validate_confirm_order_statuses_field( $key, $value ) {
		return $this->validate_field_empty( $key, $value );
	}

	// - Fields validation

	public static function doConfirmOrder( $order_id, $is_retry = false ) {
		if ( empty( $order_id ) ) {
			return false;
		}
		$order   = wc_get_order( $order_id );
		$payload = GelProximityCartPrice::getPayloadByPsIdOrder( $order_id );
		if ( ! empty( $payload ) ) {
			// Send confirm order
			$id_cart                 = get_post_meta( $order_id, '_gelproximity_order_cart_id', true );
			$id_gelprx_order_history = GelProximityOrderHistory::getIDbyPsIdCart( $id_cart );
			// Per evitare di inviare al server più volte la stessa richiesta se order confirmation viene aggiornata
			if ( empty( $id_gelprx_order_history ) || $is_retry ) {
				// Salva preventivamente il record con success = false
				if ( ! $is_retry ) {
					GelProximityOrderHistory::storeOrUpdate( $id_cart, $order_id, '' );
				}
				// Invia a GEL Proximity S2S
				$is_sandbox    = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['is_sandbox_mode'];
				$api_key_s2s   = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['api_key_s2s'];
				$merchant_code = get_option( GelProximityConfiguration::GELPRX_CONFIG_OPTION_SETTINGS_KEY )['merchant_code'];

				$urlEndUser = GelProximityConfiguration::GELPRX_API_URL_SERVER_SANDBOX;
				if ( $is_sandbox !== 'yes' ) {
					$urlEndUser = GelProximityConfiguration::GELPRX_API_URL_SERVER_PRODUCTION;
				}
				$gpApiS2s = new GelProximityApiS2s(
					$urlEndUser,
					$api_key_s2s,
					$merchant_code
				);
				$response = $gpApiS2s->confirmOrder(
					GelProximityUtility::buildOrderReference( $id_cart ),
					$order,
					$payload
				);
				// Memorizza la response (se fallisce response = false)
				GelProximityOrderHistory::storeOrUpdate( $id_cart, $order_id, $response );

				return true;
			}
		}

		return false;
	}
}