<?php
/*
   Plugin Name: GEL Proximity | La tecnologia per l’ultimo miglio
   Plugin URI: http://wordpress.org/extend/plugins/gelproximity/
   Version: 1.0.2.3
   Author: GEL Proximity
   Description: Offer your customers an alternative logistics service to home delivery! Use the over 30,000 authorized collection points in Italy, improve the UX of your customers at checkout, add your stores and start shipping nearby in Pay per Ship mode.
   Text Domain: gelproximity
   License: GPLv3
  */

/*
    "WordPress Plugin Template" Copyright (C) 2020 Michael Simpson  (email : michael.d.simpson@gmail.com)

    This following part of this file is part of WordPress Plugin Template for WordPress.

    WordPress Plugin Template is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    WordPress Plugin Template is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Contact Form to Database Extension.
    If not, see http://www.gnu.org/licenses/gpl-3.0.html
*/

$GelProximity_minimalRequiredPhpVersion = '5.2.4';

/**
 * Check the PHP version and give a useful error message if the user's version is less than the required version
 * @return boolean true if version check passed. If false, triggers an error which WP will handle, by displaying
 * an error message on the Admin page
 */
function GelProximity_noticePhpVersionWrong() {
	global $GelProximity_minimalRequiredPhpVersion;
	$html = '<div class="updated fade">' .
	        __( 'Error: plugin "GEL Proximity" requires a newer version of PHP to be running.', 'gelproximity' ) .
	        '<br/>' . __( 'Minimal version of PHP required: ',
			'gelproximity' ) . '<strong>' . esc_html( $GelProximity_minimalRequiredPhpVersion ) . '</strong>' .
	        '<br/>' . __( 'Your server\'s PHP version: ',
			'gelproximity' ) . '<strong>' . esc_html( phpversion() ) . '</strong>' .
	        '</div>';
	echo wp_kses( $html, GelProximityUtility::getHtmlAllowedTags() );
}


function GelProximity_PhpVersionCheck() {
	global $GelProximity_minimalRequiredPhpVersion;
	if ( version_compare( phpversion(), $GelProximity_minimalRequiredPhpVersion ) < 0 ) {
		add_action( 'admin_notices', 'GelProximity_noticePhpVersionWrong' );

		return false;
	}

	return true;
}


/**
 * Initialize internationalization (i18n) for this plugin.
 * References:
 *      http://codex.wordpress.org/I18n_for_WordPress_Developers
 *      http://www.wdmac.com/how-to-create-a-po-language-translation#more-631
 * @return void
 */
function GelProximity_i18n_init() {
	$pluginDir = dirname( plugin_basename( __FILE__ ) );
	load_plugin_textdomain( 'gelproximity', false, $pluginDir . '/languages/' );
}


//////////////////////////////////
// Run initialization
/////////////////////////////////

// Initialize i18n
add_action( 'plugins_loaded', 'GelProximity_i18n_init' );

// Run the version check.
// If it is successful, continue with initialization for this plugin
if ( GelProximity_PhpVersionCheck() ) {
	// Only load and run the init function if we know PHP version can parse it
	include_once( 'gelproximity_init.php' );
	GelProximity_init( __FILE__ );
}
